# coding: utf-8

"""
    ORY Keto

    A cloud native access control server providing best-practice patterns (RBAC, ABAC, ACL, AWS IAM Policies, Kubernetes Roles, ...) via REST APIs.  # noqa: E501

    The version of the OpenAPI document: v0.0.0-alpha.1
    Contact: hi@ory.sh
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest

import ory_keto_client
from ory_keto_client.models.list_ory_access_control_policy_roles_ok import ListOryAccessControlPolicyRolesOK  # noqa: E501
from ory_keto_client.rest import ApiException


class TestListOryAccessControlPolicyRolesOK(unittest.TestCase):
    """ListOryAccessControlPolicyRolesOK unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testListOryAccessControlPolicyRolesOK(self):
        """Test ListOryAccessControlPolicyRolesOK"""
        # FIXME: construct object with mandatory attributes with example values
        # model = ory_keto_client.models.list_ory_access_control_policy_roles_ok.ListOryAccessControlPolicyRolesOK()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
